@@ -26,10 +26,29 @@ import config
 import util
 import cache
 import random
-import ldap
 import socket
 import debuglog
 
+#
+# LDAP should be a soft dependency.
+#
+
+try:
+    import ldap
+    has_ldap = True;
+except ImportError:
+    has_ldap = False;
+
+#
+# selinux should be a soft dependency.
+#
+
+try:
+    import selinux
+    has_selinux = True;
+except ImportError:
+    has_selinux = False;
+
 #
 # Default empty config.
 #
@@ -84,12 +103,12 @@ class SystemDatabase(object):
     association between users and profiles.
 
     This database is stored by default in
-    $(sysconfdir)/desktop-profiles/users.xml and contains a
+    $(sysconfdir)/sabayon/profiles/users.xml and contains a
     list of users and the profile associated with each of
     those users.
 
     A profile can be reference by either its name (in which case
-    the profile is stored at /etc/desktop-profiles/$(name).zip),
+    the profile is stored at /etc/sabayon/profiles/$(name).zip),
     an absolute path or a http/file URL.
     """
     def __init__ (self, db_file = None):
@@ -97,16 +116,17 @@ class SystemDatabase(object):
 
         @db_file: a mandatory path which specifes the location
         of the database file. If only a file is specified, the
-        directory /etc/desktop-profiles is used.
+        directory /etc/sabayon is used.
         """
         if db_file is None:
             raise SystemDatabaseException(_("No database file provided"))
         elif db_file[0] != '/':
-            file = os.path.join (config.PROFILESDIR, db_file)
+            file = os.path.join (config.CONFIGDIR, db_file)
         else:
             file = db_file
         self.file = file
         self.xmlquery = None
+        self.nodes = None        # nodes from the XML file for LDAP usage.
         self.modified = 0
         dprint("New UserDatabase(%s) object\n" % self.file)
 
@@ -180,31 +200,44 @@ class SystemDatabase(object):
         return profile
 
     def __open_ldap (self):
-        nodes = self.doc.xpathEval ("/profiles/ldap")
-        if len (nodes) == 0:
-            return None
-        ldap_node = nodes[0]
+        ldap_node = self.nodes[0]
 
-        server = get_setting (ldap_node, "server", "localhost")
-        port = get_setting (ldap_node, "port", ldap.PORT, int)
+        #
+        # Get a list of uri's from the "uri" setting on the ldap node.
+        #
+        uris = get_setting (ldap_node, "uri", "ldap://localhost")
 
-        l = ldap.open (server, port)
-        
-        l.protocol_version = get_setting (ldap_node, "version", ldap.VERSION3, int)
-        l.timeout =  get_setting (ldap_node, "timeout", 10, int)
-        
-        bind_dn = get_setting (ldap_node, "bind_dn", "")
-        bind_pw = get_setting (ldap_node, "bind_pw", "")
-        if bind_dn != "":
-            l.simple_bind (bind_dn, bind_pw)
-        
-        return l
+        #
+        # Could be a comma separated list.
+        # 
+        urilist = uris.split(',')
+
+        for uri in urilist:
+            try:
+                l = ldap.initialize (uri)
+                
+                l.protocol_version = get_setting (ldap_node, "version", ldap.VERSION3, int)
+                l.timeout =  get_setting (ldap_node, "timeout", 10, int)
+                
+                bind_dn = get_setting (ldap_node, "bind_dn", "")
+                bind_pw = get_setting (ldap_node, "bind_pw", "")
+                # Bind no matter what, so we know the server's there.
+                l.simple_bind (bind_dn, bind_pw)
+                
+                return l
+            except ldap.LDAPError, error_message:
+                dprint("Couldn't bind to %s: %s\n", uri, error_message)
 
     def __ldap_query (self, map, replace):
-        nodes = self.doc.xpathEval ("/profiles/ldap/" + map)
-        if len (nodes) == 0:
+        global has_ldap
+        if not has_ldap:
             return None
-        map_node = nodes[0]
+        if not self.nodes:
+            self.nodes = self.doc.xpathEval ("/profiles/ldap/" + map)
+            if len (self.nodes) == 0:
+                has_ldap = False        # No LDAP nodes in the xml file.
+                return None
+        map_node = self.nodes[0]
         
         l = self.__open_ldap ()
         if not l:
@@ -217,13 +250,13 @@ class SystemDatabase(object):
         multiple_result = get_setting (map_node, "multiple_result", "first")
 
         if search_base == None:
-            raise SystemDatabaseException(_("No search base specified for %s"%map))
+            raise SystemDatabaseException(_("No LDAP search base specified for %s" % map))
             
         if query_filter == None:
-            raise SystemDatabaseException(_("No query filter specified for %s"%map))
+            raise SystemDatabaseException(_("No LDAP query filter specified for %s" % map))
             
         if result_attribute == None:
-            raise SystemDatabaseException(_("No result attribute specified for %s"%map))
+            raise SystemDatabaseException(_("No LDAP result attribute specified for %s" % map))
 
         if scope == "sub":
             scope = ldap.SCOPE_SUBTREE
@@ -232,7 +265,7 @@ class SystemDatabase(object):
         elif scope == "one":
             scope = ldap.SCOPE_ONELEVEL
         else:
-            raise SystemDatabaseException(_("Scope must be one of sub, base and one"))
+            raise SystemDatabaseException(_("LDAP Scope must be one of: ") + "sub, base, one")
         
         query_filter = expand_string (query_filter, replace)
         search_base = expand_string (search_base, replace)
@@ -252,7 +285,7 @@ class SystemDatabase(object):
         elif multiple_result == "random":
             val = vals[random.randint(0, len(vals)-1)]
         else:
-            raise SystemDatabaseException(_("multiple_result must be one of first and random"))
+            raise SystemDatabaseException(_("multiple_result must be one of: ") + "first, random")
 
         l.unbind ()
         
@@ -321,6 +354,7 @@ class SystemDatabase(object):
         return self.__profile_name_to_location (profile, user)
 
     def __save_as(self, filename = None):
+        global has_selinux
         """Save the current version to the given filename"""
         if filename == None:
             filename = self.file
@@ -359,6 +393,12 @@ class SystemDatabase(object):
             raise SystemDatabaseException(
                 _("Failed to save UserDatabase to %s") % filename)
 
+        if has_selinux:
+            if selinux.is_selinux_enabled() > 0:
+                rc, con = selinux.matchpathcon(filename, 0)
+                if rc == 0:
+                    selinux.setfilecon(filename, con)
+
         self.modified = 0
 
     def set_default_profile (self, profile):
@@ -450,7 +490,7 @@ class SystemDatabase(object):
     def get_profiles (self):
         """Return the list of currently available profiles.
         This is basically just list of zip files in
-        /etc/desktop-profiles, each without the .zip extension.
+        /etc/sabayon/profiles, each without the .zip extension.
         """
         list = []
         try:
@@ -514,7 +554,7 @@ class UserDatabase(SystemDatabase):
         return list
 
 class GroupDatabase(SystemDatabase):
-    """Encapsulate a user mapping
+    """Encapsulate a group mapping
     """
     def __init__ (self, db_file = None):
         if db_file is None:
@@ -599,11 +639,11 @@ def run_unit_tests ():
     db.set_profile("localuser", "groupA")
     res = db.get_profile("localuser")
     assert not res is None
-    assert res[-28:] == "/desktop-profiles/groupA.zip"
+    assert res[-20:] == "/profiles/groupA.zip"
     db.set_profile("localuser", "groupB")
     res = db.get_profile("localuser")
     assert not res is None
-    assert res[-28:] == "/desktop-profiles/groupB.zip"
+    assert res[-20:] == "/profiles/groupB.zip"
     res = db.get_users()
     print res
     db = GroupDatabase(testgroupfile)
@@ -614,11 +654,11 @@ def run_unit_tests ():
     db.set_profile("localgroup", "groupA")
     res = db.get_profile("localgroup")
     assert not res is None
-    assert res[-28:] == "/desktop-profiles/groupA.zip"
+    assert res[-20:] == "/profiles/groupA.zip"
     db.set_profile("localgroup", "groupB")
     res = db.get_profile("localgroup")
     assert not res is None
-    assert res[-28:] == "/desktop-profiles/groupB.zip"
+    assert res[-20:] == "/profiles/groupB.zip"
     res = db.get_groups()
     print res
 
